library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;


entity pomodoro is
	generic(
		ticks_per_second : natural := 10
	);
	port(
		clk        : in  std_logic;
		rst        : in  std_logic;
		toggle     : in  std_logic;
		minutes    : out std_logic_vector(5 downto 0);
		seconds    : out std_logic_vector(5 downto 0);
		active     : out std_logic;
		work_phase : out std_logic
	);
end pomodoro;

architecture logic of pomodoro is
	signal clk_1Hz        : std_logic;
	signal min_cnt        : unsigned(5 downto 0);
	signal sec_cnt        : unsigned(5 downto 0);
	signal start_counting : std_logic;
	signal counting       : std_logic;
	signal pomodoro_cnt   : unsigned(1 downto 0);
	signal work_phase_buf : std_logic;
	signal next_break_cnt : unsigned(5 downto 0);

	constant work_cnt        : unsigned(5 downto 0) := to_unsigned(25, 6);
	constant short_break_cnt : unsigned(5 downto 0) := to_unsigned(5, 6);
	constant long_break_cnt  : unsigned(5 downto 0) := to_unsigned(15, 6);
	constant zero_cnt        : unsigned(5 downto 0) := (others => '0');
begin
	active         <= counting;
	minutes        <= std_logic_vector(min_cnt);
	seconds        <= std_logic_vector(sec_cnt);
	work_phase     <= work_phase_buf;
	next_break_cnt <= long_break_cnt when pomodoro_cnt = "11" else short_break_cnt;
	divider_1Hz : work.divider generic map(ticks_per_second) port map(clk, rst, clk_1Hz);

	toggle_proc : process(clk, rst)
	begin
		if rst = '1' then
			start_counting <= '0';
		elsif rising_edge(clk) then
			if toggle = '1' then
				start_counting <= '1';
			elsif counting = '1' then
				start_counting <= '0';
			end if;
		end if;
	end process;

	timer_proc : process(clk_1Hz, rst)
	begin
		if rst = '1' then
			min_cnt        <= work_cnt;
			sec_cnt        <= (others => '0');
			pomodoro_cnt   <= (others => '0');
			work_phase_buf <= '1';
		elsif rising_edge(clk_1Hz) then
			if counting = '1' then
				if min_cnt = zero_cnt and sec_cnt = to_unsigned(1, sec_cnt'length) then
					counting       <= '0';
					work_phase_buf <= not work_phase_buf;

					sec_cnt <= (others => '0');
					if work_phase_buf = '1' then
						pomodoro_cnt <= pomodoro_cnt + 1;
						min_cnt      <= next_break_cnt;
					else
						min_cnt <= work_cnt;
					end if;
				elsif sec_cnt = zero_cnt then
					min_cnt <= min_cnt - 1;
					sec_cnt <= to_unsigned(59, sec_cnt'length);
				else
					sec_cnt <= sec_cnt - 1;
				end if;
			else
				counting <= start_counting;
			end if;
		end if;
	end process;
end logic;
